import React, { memo, useEffect, useState } from 'react';
import { Collapse, Dropdown, Icon, Menu } from 'antd';
import isEmpty from 'lodash/isEmpty';
import map from 'lodash/map';
import get from 'lodash/get';
import isArray from 'lodash/isArray';
import isEqual from 'lodash/isEqual';
import SortTree from './SortTree';
import styles from './index.less';
import { controlUpdate, usePrevious } from '../utils';
import { clearChildNodes, clearDropTarget, copyComponent, deleteComponent, getComponentConfig, isContainer, ROOT, useSelector, } from 'lingxi-editor-core';
import { getDropTargetInfo, handleSelectedStatus, onMouseOver, selectedStatus } from 'lingxi-design';
import domTreeIcons from './domTreeIcons';
const { Panel } = Collapse;
const { Item } = Menu;
const handleMenuClick = (e) => {
    switch (e.key) {
        case '1':
            return clearChildNodes();
        case '2':
        case '3':
            return copyComponent();
        case '4':
            return deleteComponent();
    }
};
function renderMenu(domKey, isOnlyNode, isClear) {
    const isRoot = domKey === ROOT;
    return (React.createElement(Menu, { onClick: handleMenuClick },
        isClear && React.createElement(Item, { key: 1 }, "\u6E05\u9664"),
        !isRoot && !isOnlyNode && React.createElement(Item, { key: 3 }, "\u590D\u5236"),
        !isRoot && React.createElement(Item, { key: 4 }, "\u5220\u9664")));
}
;
function getIcon(name) {
    if (get(domTreeIcons, `${name}`))
        return get(domTreeIcons, `${name}`);
    return domTreeIcons.Layout;
}
;
/**
 * 渲染页面结构节点
 * @returns {*}
 */
function renderHeader(isUnfold, props, isSelected, isHovered, setIsUnfold, componentName, childNodes, nodePropsConfig) {
    const { specialProps, specialProps: { key } } = props;
    const selectedColor = '#5E96FF';
    const unSelectedColor = '#555555';
    const selectedBGColor = '#F2F2F2';
    const hoveredBGColor = '#F1F1F1';
    const color = isSelected ? selectedColor : unSelectedColor;
    let propName = props.propName;
    let isNodePropsRoot = false;
    if (nodePropsConfig && !propName) {
        propName = Object.keys(nodePropsConfig)[0];
        isNodePropsRoot = true;
    }
    return (React.createElement("div", { style: { backgroundColor: isSelected ? selectedBGColor : isHovered ? hoveredBGColor : '#0000' }, className: styles['header-container'] },
        React.createElement("div", { onClick: () => handleSelectedStatus(null, isSelected, specialProps, propName), onMouseOver: (e) => onMouseOver(e, isNodePropsRoot || propName ? `${key}${propName}` : key, isSelected), style: { display: 'flex', flex: 1, alignItems: 'center', color } },
            React.createElement(Icon, { className: isUnfold ? styles.rotate90 : '', style: {
                    padding: 5,
                    fontSize: 16,
                    transition: 'all 0.2s',
                    color: isNodePropsRoot || !isEmpty(childNodes) ? unSelectedColor : '#0000',
                }, type: "caret-right", onClick: (event) => {
                    event.stopPropagation();
                    setIsUnfold(!isUnfold);
                } }),
            React.createElement(Icon, { component: getIcon(componentName), style: { marginRight: 7 } }),
            React.createElement("span", null, componentName)),
        isSelected &&
            React.createElement(Dropdown, { trigger: ['click'], overlay: renderMenu(key) },
                React.createElement(Icon, { component: getIcon('more'), style: { color } }))));
}
/**
 * 渲染子组件或者属性节点
 * @returns {Array|*}
 */
function renderSortTree(props, isUnfold, componentName, nodePropsConfig, childNodes) {
    const { specialProps, propName, nodeProps } = props;
    if (childNodes && isArray(childNodes) || !childNodes && !nodePropsConfig) {
        return (React.createElement(SortTree, { isFold: !isUnfold, childNodes: childNodes && childNodes || [], propName: propName, specialProps: specialProps, nodeProps: nodeProps, componentName: componentName }));
    }
    /**
     * 处理属性节点子组件
     */
    return map(nodePropsConfig, (nodeProps, propName) => {
        // const propKey = `${key}${propName}`;
        const propChildNodes = childNodes && childNodes[propName] || [];
        return React.createElement(SortItem, Object.assign({}, props, { propChildNodes: propChildNodes, specialProps: specialProps, propName: propName, key: propName, nodeProps: nodeProps }));
    });
}
export const stateSelector = ['selectedInfo', 'hoverKey', 'componentConfigs'];
function SortItem(props) {
    const { specialProps: { key, parentPropName, parentKey, domTreeKeys }, isFold, propName, propChildNodes, } = props;
    const { selectedInfo, hoverKey, componentConfigs } = useSelector(stateSelector, (prevState, nextState) => controlUpdate(prevState, nextState, key));
    const { selectedKey, domTreeKeys: nextSDTKeys, propName: selectedPropName } = selectedInfo || {};
    const { isHovered, isSelected } = selectedStatus(propName ? `${key}${propName}` : key, hoverKey, selectedPropName ? `${selectedKey}${selectedPropName}` : selectedKey);
    const vDom = componentConfigs[key];
    const { childNodes: vDomChildNodes, componentName } = vDom || {};
    const { fatherNodesRule, nodePropsConfig } = getComponentConfig(componentName);
    const childNodes = propChildNodes || vDomChildNodes;
    const [isUnfold, setIsUnfold] = useState(!isEmpty(childNodes));
    // 保存子组件dom
    const prevChildNodes = usePrevious(childNodes);
    const prevSDTKeys = usePrevious(nextSDTKeys);
    //新添加组件展开
    useEffect(() => {
        if (!isUnfold && !isEqual(prevChildNodes, childNodes)) {
            setIsUnfold(true);
        }
    }, [prevChildNodes, childNodes, isUnfold]);
    // 父节点折叠当前节点是展开的就折叠当前节点
    useEffect(() => {
        if (isFold && isUnfold)
            setIsUnfold(false);
    }, [isFold, isUnfold]);
    if (!componentName)
        return null;
    if (!isEqual(prevSDTKeys, nextSDTKeys) && nextSDTKeys && !isUnfold && nextSDTKeys.includes(key)) {
        setIsUnfold(true);
    }
    const isContainerComponent = isContainer(componentName);
    return (React.createElement("div", { className: styles['sort-item'], id: key, "data-special": JSON.stringify({ key, parentPropName, parentKey }), "data-farules": fatherNodesRule && JSON.stringify(fatherNodesRule), "data-name": componentName, onDragEnter: (e) => {
            //如果目标组件为非容器组件就重置目标容器信息
            if (!isContainerComponent)
                return clearDropTarget();
            let propNameResult = propName;
            //如果当前目标是多属性节点容器，获取容器的最后一属性节点作为目标容器
            if (!propNameResult && nodePropsConfig) {
                propNameResult = Object.keys(nodePropsConfig).pop();
            }
            getDropTargetInfo(e, domTreeKeys, key, propNameResult);
        } },
        renderHeader(isUnfold, props, isSelected, isHovered, setIsUnfold, propName || componentName, childNodes, nodePropsConfig),
        isContainerComponent && React.createElement(Collapse, { activeKey: isUnfold ? ['1'] : [], style: { marginLeft: 24 }, bordered: false },
            React.createElement(Panel, { showArrow: false, key: '1', header: React.createElement("div", null), style: { border: 0, backgroundColor: '#fff' } }, renderSortTree(props, isUnfold, componentName, nodePropsConfig, childNodes)))));
}
export default memo(SortItem);
